require_relative '../lib/showmd/preview_generator'
require 'minitest/autorun'
require 'minitest/spec'
require 'minitest/pride'

module Showmd
  class PreviewGeneratorSpec < MiniTest::Test

    INVALID_FILE = File.expand_path(File.join(File.dirname(__FILE__), "fixtures", "doesnt_exist.html"))
    TEST_OUTPUT_PATH = File.expand_path(File.join(File.dirname(__FILE__), "fixtures", "preview.html"))
    TEST_TEMPLATE_PATH = File.expand_path(File.join(File.dirname(__FILE__), "fixtures", "template.html"))
    VALID_MD_FILE = File.expand_path(File.join(File.dirname(__FILE__), "fixtures", "valid.md"))

    describe "read_md_file" do
      it "should read a valid file" do
        preview_generator = Showmd::PreviewGenerator.new(VALID_MD_FILE)
        preview_generator.read_md_file.must_match "*Here we have some markdown!*"
      end

      it "should raise if file is invalid" do
        lambda do
          Showmd::PreviewGenerator.new(INVALID_FILE).read_md_file
        end.must_raise(Showmd::InputError)
      end
    end

    describe "build_html_file" do
      before do
        @preview_generator = Showmd::PreviewGenerator.new(VALID_MD_FILE)
        @preview_generator.read_md_file
        @preview_generator.output_path = TEST_OUTPUT_PATH
        @preview_generator.template_path = TEST_TEMPLATE_PATH
      end
      it "should read from the html template" do
        @preview_generator.build_html_file

        File.exist?(@preview_generator.output_path).must_be :==, true
        File.delete(@preview_generator.output_path)
      end

      it "should contain the markdown string" do
        @preview_generator.build_html_file

        File.read(@preview_generator.output_path).must_match "*Here we have some markdown!*"
        File.delete(@preview_generator.output_path)
      end
    end

    describe "generate" do
      it "should call methods to read, build and open" do
        showmd = MiniTest::Mock.new
        def Launchy.open(x);;end
        preview_generator = Showmd::PreviewGenerator.new(VALID_MD_FILE).generate

        showmd.expect :read_md_file, true
        showmd.expect :build_html_file, true
        showmd.expect :open_in_browser, true
      end
    end
  end
end
